var Timer = null;

(function($) {
	
Timer = Class.extend({
	/**
	 * Constructor
	 */
    init: function (options) {
        this.defaults = {
        	currentPage: 1,
        	useNumberKeys: true
        };
        
        this.options = $.extend(this.defaults, options);
        
        this.groupsInputField = "input[name='amountGroups']";
        this.roundsInputField = "input[name='amountRounds']";
        this.minutesField = "input[name='roundMinutes']";
        this.secondsField = "input[name='roundSeconds']";
        this.amountGroups = 0;
        this.amountRounds = 0;
        this.roundMinutes = 0;
        this.roundSeconds = 0;
        this.roundTime = 0;
        this.startTime;
        this.times = [];
        this.newTimer;
        this.startTimer = false;
        this.buzzerClicks = 0;
        this.convertedTime = {};
        this.contentFadeTime = 200;				// In milliseconds
        this.timerSettingsErrorMsg = "Die Rundezeit kann nicht 0 sein. Die Anzahl der Runden & die Anzahl der Gruppen können nicht Null sein.";
        this.timerAnimation = null;
        this.animationDiv = null;
        this.animationStarted = false;
        this.rotationDegrees = 0;    
        this.helpText = "Für eine Gruppe (Einzelspieler) die A-Taste zum Start der ersten Runde drücken. " +
        		"Jeder weitere Push auf A stoppt die laufende Runde und misst die Zeit."+
        		"<br />Bei zwei Gruppen nacheinander erst die A-Taste, danach die Enter-Taste zum Start der ersten Runden drücken. " +
        		"Jeder weitere Push auf diese Tasten stoppt die laufenden Runden und misst die Zeit.";
        this.defaultGroupKeycodes = [65, 13]; 	// "a", "Enter"
    	
        this.initialize();
    },
    
    initialize: function() {
    	this.showCurrentPage(this.defaults.currentPage);
    },
    
    showCurrentPage: function(page) {
    	var self = this;
    	var html = "";
    	
    	$('#pageContent').fadeOut(this.contentFadeTime);
    	
    	switch (page) {
    	case 1:
    		html = this.getStartPageContent();
    		break;
    	case 2:
    		html = this.getTimerPage();
    		break;
    	case 3:
    		html = this.getChartPage();
    		break;
    	default:
    		html = this.getStartPageContent();
		break;
    	}
    	setTimeout(function() {
    		document.getElementById('pageContent').innerHTML = "";
        	document.getElementById('pageContent').innerHTML = html;
        	$('#pageContent').fadeIn(this.contentFadeTime);
    	}, self.contentFadeTime);
    	
    	setTimeout(function() {
    		self.setEventListeners(page);
    		if(page == 3)
    			self.drawChart();
    	}, self.contentFadeTime + 100);
    },
    
    getStartPageContent: function () {
    	return '<div id="startPage" class="toggleContent">'+
					'<div class="pageHeader">'+
						'<span class="contentTitle">Willkommen</span>'+
					'</div>'+
					'<div class="mainContent">'+
						'<div class="pageDescription box">'+
						'Willkommen beim Teachtoday Timer. Teste dein Zeitgefühl! Bitte gib\' die Anzahl der Runden und Gruppen ein. '+
						'Wähle ausserdem die Dauer der Runde.'+
						'</div>'+
						'<span class="contentTitle">Timer Einstellungen</span>'+
						'<div class="startContent">'+
							'<div class="timerForm box">'+
							'<form>'+
								'<div class="formRow">'+
									'<h2>Gruppen</h2>'+
									'<label>Anzahl der Gruppen: </label>'+
									'<input class="numberInput" name="amountGroups" type="number" min="1" max="2" />'+
								'</div>'+
								'<div class="formRow">'+
									'<h2>Runde</h2>'+
									'<label>Anzahl der Runden: </label>'+
									'<input class="numberInput" name="amountRounds" type="number" min="1" max="20"/>'+
								'</div>'+
								'<div>'+
									'<h2>Zeit</h2>'+
									'<label>Minuten: </label>'+
									'<input class="numberInput" name="roundMinutes" type="number" min="0" />'+
								'</div>'+
								'<div class="formBottomRow">'+
									'<label>Sekunden: </label>'+
									'<input class="numberInput" name="roundSeconds" type="number" min="0" max="59" />'+
								'</div>'+
							'</form>'+
							'</div>'+
							'<button id="start" class="btNext bt">Starten</button>'+
							'<div class="timerValuesError"></div>'+
						'</div>'+
					'</div>'+
				'</div>';
    },
    
    getTimerPage: function () {
    	var html = '<div id="timerPage" class="toggleContent"><div class="pageHeader"><span class="contentTitle">Prüfe dein Zeitgefühl</span></div>'+
    				'<div class="mainContent">'+
    				'<div class="pageDescription box">'+this.helpText+'</div>'+
    				'<div class="clock"></div>';
    	for(var i = 0; i < this.amountGroups; i++) {
    		html = html + '<section class="group">'+
    							'<h1>Gruppe '+(i+1)+'</h1>'+
    							'<div class="fontRNumber roundGroup_'+i+'"><span class="currentRound">Aktuellen Runde: </span><span id="rn_'+i+'" class="roundNumber"><b>0</b></span> / '+this.amountRounds+'</div>'+
    							// For testing with buttons -> '<button id="btStop_'+i+'" class="btStopRound bt" group="'+i+'">Runde Starten</button>'+
    					   '</section>';
    		if(i == 0) {
    			html = html + '<div id="timerAnimation"><div id="bottomLayer" class="animationLayer"><img src="img/ring_layer_1.png" /></div>'+
    			'<div id="upperLayer" class="animationLayer"><img src="img/ring_layer_2.png" /></div></div>';
    		}
    	}
    	html = html + '<div class="timerBottom"><button class="btResults bt btLeft">Ergebnisse</button>'+
    	'<button class="btStopTimer bt btLeft">Timer Stoppen</button><button class="btBack bt">Neu Starten</button></div></div></div>';
    	return html;
    },
    
    drawChart: function () {
    	TimeChart = new TimeChart({
    		chartData: this.times,
    		roundMinutes: this.roundMinutes,
    		roundSeconds: this.roundSeconds,
    		amountRounds: this.amountRounds
    	});
    },
    
    getChartPage: function () {
    	return '<div id="chartPage" class="toggleContent"><div class="pageHeader"><span class="contentTitle">Auswertung</span></div>'+
		'<div class="mainContent"><div class="pageDescription box">Rundenzeiten der Gruppe <span class="groupID">1</span>.</div>'+
    	'<div id="chart"></div><div id="chartGroups"></div><button class="btStartPage bt">Neu Starten</button></div></div>';
    },
    
    setEventListeners: function (page) {
    	var self = this;
    	
		switch (page) {
		case 1:
			 // Input type number for IE, with polyfill
			$('.numberInput').inputNumber();
			$('div.number-spin-btn-container').last().css('margin-top', '4px');
			
			$('.btNext').click(function() {
    			self.setTimerValues();
    			if(self.startTimer) {
            		self.showCurrentPage(self.defaults.currentPage + 1);
            	}
    			else
        			$('.timerValuesError').text(self.timerSettingsErrorMsg);
			});
			break;
		case 2:
			var groups = [];
			
			for(var i = 0; i < this.amountGroups; i++) {
	    		this.times.push(new Array());
	    		groups.push({ clicks:0, round:2, previousTime: self.startTime, startTime: 0, started: false });
	    	}
			
			document.onkeydown = function (event) {
				self.timerClickCallback(groups, self.getGroupIndexFromKeyCode(event.keyCode));
			};
			
			// For testing with buttons
			/*
	    	$('.btStopRound').on('click', function() {
	    		group = $(this).attr('group');
	    		self.timerClickCallback(groups, group);
	    	});
			*/
	    	$('.btStopTimer').click(function() {
	    		clearTimeout(self.newTimer);
	    		clearInterval(self.timerAnimation);
	    		$('.btStopRound').off();
	    		$('.btResults').fadeIn(self.contentFadeTime);
	    		$('.btResults').click(function() {
		    		self.showCurrentPage(self.defaults.currentPage + 1);
		    	});
	    	});
	    	
	    	$('.btBack').on('click', function() {
	    		location.reload();
	    	});
			break;
		case 3:
			$('.btStartPage').on('click', function() {
				location.reload();
	    	});
			break;
		}
    	this.defaults.currentPage = page;
    },
    
    setTimerAnimation: function (dg) {
    	var self = this;
    	
    	this.timerAnimation = setInterval(function() {
    		if(self.rotationDegrees <= 360)
    			self.rotationDegrees = self.rotationDegrees + dg;
        	else
        		self.rotationDegrees = dg;
            
    		self.animationDiv.style.webkitTransform = 'rotate('+self.rotationDegrees+'deg)';
    		self.animationDiv.style.mozTransform    = 'rotate('+self.rotationDegrees+'deg)';
    		self.animationDiv.style.msTransform     = 'rotate('+self.rotationDegrees+'deg)';
    		self.animationDiv.style.oTransform      = 'rotate('+self.rotationDegrees+'deg)';
    		self.animationDiv.style.transform       = 'rotate('+self.rotationDegrees+'deg)';
    	}, 80);
    },
    
    setVisibleTimer: function () {
    	var self = this;
    	this.startTime = new Date();
    	this.newTimer = setTimeout(t, 100);
    	
    	function t() {
    		var currentTime = self.convertTime(new Date() - self.startTime);
    		document.querySelector('.clock').textContent = ("0"+currentTime.minutes).slice(-2)+":"+("0"+currentTime.seconds).slice(-2);
    		self.newTimer = setTimeout(t, 100);
    	}
    },
    
    timerClickCallback: function (groups, groupIndex) {
    	var self = this;
    	
    	if( groupIndex > this.amountGroups || groupIndex < 0 )
    		return false;
    	
    	var currGroup = groups[groupIndex];
    	
    	if(currGroup.started) {
	    	this.buzzerClicks++;
	    	currGroup.clicks++;
			
			if(currGroup.clicks < this.amountRounds) {
				this.convertedTime = this.convertTime(new Date() - currGroup.previousTime);
				this.times[groupIndex].push(this.convertedTime);
				
				if(currGroup.round <= this.amountRounds)
					$('#rn_'+groupIndex).html("<b>"+currGroup.round+"</b>");
				
			}
			else if(currGroup.clicks == this.amountRounds) {
				this.convertedTime = this.convertTime(new Date() - currGroup.previousTime);
				this.times[groupIndex].push(this.convertedTime);
				
				$('.roundGroup_'+groupIndex).css('color', '#12BFE7');
				
				if( this.buzzerClicks == (this.amountGroups*this.amountRounds) ) {
					clearTimeout(this.newTimer);
					clearInterval(self.timerAnimation);
					$('.btResults').fadeIn(this.contentFadeTime);
		    		$('.btResults').click(function() {
			    		self.showCurrentPage(self.defaults.currentPage + 1);
			    	});
				}
			}
			currGroup.previousTime = new Date();
			currGroup.round++;
    	}
    	else {
    		$("#btStop_"+groupIndex).text("Nächsten Runde");
    		currGroup.startTime = new Date();
    		currGroup.previousTime = currGroup.startTime;
    		currGroup.started = true;
    		$('#rn_'+groupIndex).html("<b>"+(currGroup.round - 1)+"</b>");
    	}
    	
    	if(!this.animationStarted) {
    		this.animationDiv = document.getElementById("upperLayer");
    		this.setTimerAnimation(2);
    	}
    	this.animationStarted = true;
    },
    
    setTimerValues: function () {
    	this.roundMinutes = $(this.minutesField).val();
    	this.roundSeconds = $(this.secondsField).val();
    	
    	var minutesMSecs = this.roundMinutes * 60 * 1000;
    	var secondsMSecs = this.roundSeconds * 1000;
    	
		this.amountGroups = $(this.groupsInputField).val();
		this.amountRounds = $(this.roundsInputField).val();
		this.roundTime = minutesMSecs + secondsMSecs;
		
		this.checkTimerValues();
    },
    
    checkTimerValues: function () {
    	if(this.amountGroups < 1)
    		this.startTimer = false;
    	
    	else if(this.amountRounds < 1)
    		this.startTimer = false;
    	
    	else if(this.roundTime < 1)
    		this.startTimer = false;
    	
    	else
    		this.startTimer = true;
    },
    
    convertTime: function (msecs) {
    	var seconds = Math.floor(msecs / 1000);
		var min = Math.floor(seconds / 60);
		var sec = seconds % 60;
		var convertedTime = {minutes: min, seconds: sec};
		
		return convertedTime;
    },
    
    getGroupIndexFromKeyCode: function (kcode) {
    	var gIndex = -1;
    	
    	if(this.options.useNumberKeys) {
    		gIndex = kcode - 49;
    	}
    	else {
    		for(var i = 0; i < this.defaultGroupKeycodes.length; i++) {
    			if(this.defaultGroupKeycodes[i] == kcode) {
    				gIndex = i;
    				break;
    			}
    		}
    	}
    	return gIndex;
    }
    
});

})(jQuery);